########################################################
#                                                      #
#                   PSYC201 R Lab                      #
#          Lesson 17 - Resampling methods              #
#                                                      #
########################################################

library(PSYC201)
library(moments)
library(ggplot2)
library(gridExtra)

################################
# 17.1 - Randomization tests   #
################################

# Recall that randomization tests are methods of testing the null hypothesis with fewer assumptions
# In these tests, we shuffle the data such that the null hypothesis applies
# Then we can continue shuffling to find a distribution of our test statistic
# If our actual test statistic is unexpected given the permuted test distribution...
# Then we have evidence of a statistically significant effect

# In this way, randomization tests all have the following structure:
#  1) Calculate the real test statistic T from the data
#  2) Shuffle the data to align with the null hypothesis
#  3) Calculate a test statistic T* from the shuffled data
#  4) Repeat steps 2 & 3 a good number of times
#  5) Form a distribution of the resampled test statistic
#  6) Find where the real statistic T lies in this distribution
#     ... if it is in the most extreme 5% (for alpha=0.05), it's a significant effect

# Now R does not have many good functions for resampling...
# Their syntax tends to be relatively flexible because these tests are flexible...
# And therefore the syntax tends to be opaque

# However, the only really tricky part about all of this is step 2
# Defining the shuffled distribution under the null hypothesis is a bespoke task
# But once this is defined, the other steps are relatively easy to do

# So let's start with a basic example: significance testing of rank correlation
# Recall in a prior lesson we discussed rank correlation:
#  Looking at whether a monotonic relationship exists instead of a linear one
# But last week we mentioned we couldn't really get a p-value out of that by classic methods
# (We could we cor.test, but that runs a separate test we haven't learned about)
# Now with randomization we can make our own tests!

# Let's start with some fake data - is there a relationship between x and y here?
set.seed(9876)
dat = data.frame(x = c(rnorm(20, 0, 1.2), -5, 5), y = c(rnorm(20, 0, 1.2), -5, 5))
with(dat, cor(x,y)) # r = 0.55
summary(lm(y~x, dat))

# Great, significant p-value, right!
# Well, let's look at the data...
qplot(x,y,data=dat)

# This might be driven by the high and low ends here...
# How's the Spearman rank correlation?
rho = with(dat, cor(x,y,method='spearman'))
rho # rho = 0.22

# Quite a bit lower... but is this now statistically significant?
# We can't use classical statistical tests for this... but we can use randomization
# We just did step 1: we know the actual value of the rank correlation (0.22)

# But onto step 2: getting a null hypothesis distribution
# Now we know that under the null hypothesis, the expected value of the spearman correlation would be 0
# But that won't ever be exactly the case - try running this a few times:
cor(rnorm(30),rnorm(30),method='spearman')

# Instead we need to ask how much spread in rank correlations should there be under the null hypothesis?
# And thus we need to make a null hypothesis from our data
# To do this, we can make an assumption:
#  If there is structure in the data, then it matters which xs go with which ys
#  So we can destroy that structure by mixing them up
#  And so if we randomly mix around the 'y' values, we've destroyed the structure in the data we want to test for
# We can write a function that will take in the data and mess it up for us:
shufdat = function(d) {
  d$y = sample(d$y,length(d$y))
  return(d)
}

# Now for step 3: if we test the rank correlation with this data, we will get something else
with(shufdat(dat), cor(x,y,method='spearman'))

# Each of these is a possible rank correlation under the null hypothesis
# We get the full null hypothesis with step 4: repeating this over and over
# We'll use the replicate function to get 1000 of these guys:
null.rhos = replicate(1000, with(shufdat(dat),cor(x,y,method='spearman')))

# Step 5: how does this distribution look?
qplot(null.rhos)

# Finally, for step 6 we want to know where the real test statistic falls in the distribution
# You can eyeball this and see that 0.22 is reasonably within this distribution, so we probably won't find it significant...
# But we can get a direct p-value

# First we need to make an Empirical Cumulative Distribution Function (ecdf)
# The ecdf() function takes a distribution and gives you back a function that tells you where another number falls in that distribution
# So we make our ecdf:
null.rho.ecdf = ecdf(null.rhos)

# And then we find at what percentile our real T statistic falls at:
null.rho.ecdf(rho)

# This means that ~84.7% of our null hypothesis T* values fell below the T statistic
# (Because it's random, the number you get might be different, but similar to this value)
# To get a p-value, let's recall the definition of a p-value from the first term:
# The probability that, under the null hypothesis, we get the data as or more extreme than we actually found
# So we have a 15.3% chance of getting something more extreme on the upper tail
1 - null.rho.ecdf(rho)

# Which is a 30.6% chance of getting something as or more extreme on both sides:
2*(1 - null.rho.ecdf(rho))

# So p ~ 0.306

# But note that we only took 1000 samples... this isn't every possible permutation
# So if we run this test again, we'll get a slightly different result:
null.rhos = replicate(1000, with(shufdat(dat),cor(x,y,method='spearman')))
null.rho.ecdf = ecdf(null.rhos)
2*(1 - null.rho.ecdf(rho))

# Often 1000 samples will be enough to find statistical significance, especially if all you want is p < 0.05
# (If you're getting 0.045 < p < 0.055, you may need refinement - but in this case it's close to the edge anyhow)
# So it's fine for quick and dirty randomization tests, but for real tests you may want more (e.g., 10,000)

# Now note that the bespoke part of this analysis is defining the test statistic and shuffling
# Repeated sampling and testing the p-value are fairly automatic
# And so the PSYC201 package has a way of doing this automatically

# The function permute.test takes three arguments:
#  1) The original data
#  2) The statistic function, which takes a data frame and returns the test statistic
#  3) The shuffle function, which takes a data frame and shuffles it

# For this, we'll need to make a function that calculates the Spearman correlation from a data frame (#2):
spearcor = function(d) {
  return(with(d,cor(x,y,method='spearman')))
}

# But we already have the data (#1 - dat) and shuffle function (#3 - shufdat)
rho.ptest = permute.test(dat,spearcor,shufdat)
rho.ptest

# There are two things you can do with the output from this function:

# First, you can visualize the null distribution and where the real statistic sits with plot():
plot(rho.ptest)

# Next, you can get the p-value of the test out with a simple command that we know too well:
get.p(rho.ptest)

# You can also do this with more samples for a tighter estimate by setting giving it a different n argument:
rho.ptest = permute.test(dat,spearcor,shufdat,n=5000)
rho.ptest

# Another benefit of randomization tests is that you can run tests on any arbitrary statistic so long as you have the appropriate null hypothesis
# For instance, let's look back at the fooddat data from last lesson:
load(url('http://vulstats.ucsd.edu/RAssignData/L16_data.RData'))
head(fooddat)

# Here we were asking whether people's rating of brussel sprouts and broccoli were different
# And we could do this with a Wilcoxon test:
wilcox.test(fooddat$Brussel, fooddat$Broccoli)

# But remember these distributions didn't quite look the same:
grid.arrange(qplot(fooddat$Brussel),qplot(fooddat$Broccoli))

# Skewness seems like a promising thing that might differ between the two groups...
skewness(fooddat$Brussel)
skewness(fooddat$Broccoli)

# So let's go about testing it!
# First thing is we must define our null hypothesis
# If there isn't a difference between the skewness, then the difference in the skew of the two groups should be zero (on average)
# However, we need to respect that there may be differences in the mean and standard deviation...
# So we don't want any differences between the mean or variance to influence our test

# How do we set both foods on equal footing then?
# Z-score!
# This method makes the mean and standard deviation of both distributions equivalent (so they won't muck with our results)
# But keeps the original skewness:
z.brus = (fooddat$Brussel - mean(fooddat$Brussel))/sd(fooddat$Brussel)
z.broc = (fooddat$Broccoli - mean(fooddat$Broccoli)) / sd(fooddat$Broccoli)

skewness(z.brus)
skewness(z.broc)

# Next lets stack our data into a single frame
fd = data.frame(ZScore = c(z.brus,z.broc), Food = rep(c('ZBrus','ZBroc'),times=c(length(z.brus),length(z.broc))))
head(fd)

# Now let's calculate the difference in skew:
skewdiff = function(d) {
  skews = with(d,tapply(ZScore,Food,skewness))
  return(skews[1] - skews[2])
}

# So step 1: our real T is:
skewdiff(fd)

# Now onto step 2: shuffling
# Again, we don't need to keep any structure in the data, since we're assuming no difference between the groups
# Thus we just shuffle score in place with our shuffle function
shuffood = function(d) {
  d$ZScore = sample(d$ZScore,length(d$ZScore))
  return(d)
}

# Then we can do the rest of the steps using permute.test:
permute.test(fd, skewdiff, shuffood, n=5000)

# Well, p is really close to 0 (it can never be 0, because there is a lower limit to the test)
# The skewness is about as extreme as you can get
# But now we have statistical justification that the difference between the groups clearly isn't chance

################################
# 17.2 - Bootstrapping         #
################################

# The other sort of resampling test we need to learn to do is bootstrapping
# As a simple case, what if we want to know the median of an assymmetric distribution?
# For instance, a certain lognormal distribution:
qplot(seq(1,10,by=.05),dlnorm(seq(1,10,by=.05),2),geom='line')

# Now we could do this by taking tons and tons of samples and taking the median...
# It will eventually converge to approximately exp(2)
median(rlnorm(20000,2))

# But for realistic data, you never get 20000 samples...
# So what if we only had 50?
lndist = rlnorm(50,2)
median(lndist)

# Now we might be a bit off due to the small number of samples
# So let's try to get a confidence interval...
# But we don't know what the sampling distribution of the median will be...

# Let's try bootstrapping one!
# Recall, in boostrapping, we assume that our original data is the actual distribution of the data that exists
# We use this assumption to run synthetic replications of our experiment from that distribution of data
# So we resample from the original data, and recompute the test statistic
# Once we repeat this over and over, we get a distribution of test statistics
# From which we can draw a confidence interval
# (Note we need to do this WITH replacement)

resamp = function(dat) {
  return(sample(dat,length(dat),replace=TRUE))
}

median(resamp(lndist))

# And we redo this a lot...
meds = replicate(1000,median(resamp(lndist)))

# What do we get?
qplot(meds)

# And we can get the 95% confidence interval by looking at where the lowest and highest 2.5% show up
quantile(meds,.025) # Lower bound
quantile(meds,.975) # Upper bound

# But we also should check for bias in our bootstrap sample
# Bias is the difference between the mean value in the sample and the actual test statistic
# So we have:
median(lndist)
mean(meds)

# And thus the bias is:
mean(meds) - median(lndist)

# This isn't too bad...
# But if there's lots of bias, then the CI's can be biased too
# So always watch for bias... especially in skewed distributions where bias is more likely to occur
# There are ways to correct for bias in bootstrapping, but they have their own issues and are beyond the scope of this course

# Now, of course, once you've defined your test statistic and shuffle function, Rfunctions has an easy way to do this
# bootstrap.CI works very similarly to permute.test
# It takes at a minimum two arguments: the data and a test statistic
# By default, bootstrap.CI will resample assuming you are keeping the structure fully intact with case resampling...
#  in this case there is no structure so it just resamples from the existing data (just like the resamp function above)
bootstrap.CI(lndist,median)

# If you store this to a variable, you get out a vector with the CI, mean of the distribution, and bias
med.bci = bootstrap.CI(lndist,median)
med.bci

# And you can also set the number of bootstrap resamples with the n parameter:
bootstrap.CI(lndist,median,n=5000)

# But note that it won't change all that much...
# With bootstrapping, the limitation tends to be in your original data

# Finally, you can change the confidence interval with the conf.int parameter
bootstrap.CI(lndist,median,conf.int = .99)

# We can also do smooth resampling on this data
# ... but we will need to redefine our sampler to add variance
# Let's define a some gaussian noise at 1/10th of the width of the original distribution:
resamp.smooth = function(dat) {
  return(sample(dat,length(dat),replace=TRUE) + rnorm(length(dat),0, sd(dat)/10))
}

# Now bootstrapping is as easy as throwing everything into boostrap.CI
bootstrap.CI(lndist,median,resamp.smooth)

# This doesn't change much here (we already have 50 data points) but can be helpful in other situations

# But what about more complex bootstrap designs?
# Say, for instance, you wanted to find the confidence interval on the standard deviation of the error around an ANOVA...
# We don't know how to do this

# So let's do this for the standard deviation of the error from warpbreaks
# First we need to define a way of getting the standard deviation out
summary(lm(breaks~wool*tension,data=warpbreaks)) # sigma = 10.94

# To cut things short - here's a function that does it for you (since calculating it by hand would be painful)
wbSig = function(dat) {
  dat.lm = lm(breaks~wool*tension,data=dat)
  return(summary(dat.lm)$sigma)
}

wbSig(warpbreaks)

# But now the tricky part is defining the shuffling
# Here we can't just randomly pick data out however we like, because not all data points are equal
# Instead we have to replicate the experiment assuming our empirical data is the way the world is
# So we have to redraw everything in condition A-H from the subset of the data we oberved from A-H
# And likewise, redraw B-H from B-H, and so forth

# This requires some trickiness with coding to do 
# (But bonus points if you can figure it out! Even more if you can do it without loops!)
# Instead, we'll use a function in the PSYC201 package called resample.data()
# This function by default samples everything in place:
#  resampling rows of data (to keep correlational structure) within each cell of a full factorial interaction on the factors
# So this will resample warpbreaks the way we described it above:
resample.data(warpbreaks)

# By default, this includes every single one of your factors as a category
# If you want to cut this down to prespecified factors, you can provide them in the 'const.categories' argument
# For instance, if we want to resample only in wool, we would write:
resample.data(warpbreaks, const.categories = 'wool')

# Now we can bootstrap this (using the full model)
# When using a data frame though, resample.data without any arguments is the default method
bootstrap.CI(warpbreaks,wbSig)

# Now this gives us a relatively good estimate of how observations deviate around the predictions in the population

# As a final note about bootstrapping:
# It's only as good as your original data
# You must assume that your data is representative of the population
# So if you have an odd sample, you are making a poor assumption
# And thus bootstrapping won't give you the right answer
# Odd samples become more likely with small sample sizes
# So don't think of resampling methods as a way of getting around collecting data

# And that's the gist of (basic) resampling
# Note that the technique is easy...
# The difficult part is understanding how to shuffle your data to preserve the information you want and destroy the rest
# But that's the custom part of resampling...
# Doing this right takes some thought about your data and test statistic, and a bit of practice

