# Functions for retrieving statistical test information easily
# Plus z-test function (limited functionality)

trim.whitespace = function (x) {
  if(length(x) > 1) {return(sapply(x,trim.whitespace))}
  return(gsub("^\\s+|\\s+$", "", x))
}

# Make the generic 'get.p' function to retrieve p-values from various statistical data structures
# Also generic 'get.stat' function to return various test statistics
get.p = function(x,...) UseMethod('get.p',x)
get.stat = function(x,...) UseMethod('get.stat',x)
get.df = function(x,...) UseMethod('get.df',x)
get.ci = function(x,...) UseMethod('get.ci',x)

get.p.default = function(x,...) {writeLines('get.p requires htest, lm, aov, or anova object')}
get.stat.default = function(x,...) {writeLines('get.stat requires htest, lm, aov, or anova object')}
get.df.default = function(x,...) {writeLines('get.df requires htest, lm, aov, or anova object')}
get.ci.default = function(x,...) {writeLines('get.p requires htest or lm object')}

# For hypothesis tests, just returns the p-value or test statistic in the slot
get.p.htest = function(obj) {return(obj$p.value)}
get.stat.htest = function(obj) {return(obj$statistic)}
get.df.htest = function(obj) {
  if(is.na(obj$parameter)) {
    writeLines(paste('No degrees of freedom for:',obj$method))
    return(invisible(NULL))
  }
  return(obj$parameter)
}
get.ci.htest = function(obj) {return(obj$conf.int)}
get.ci.bootstrapped = function(obj) {return(obj$conf.int)}

# For anova objects, return all p values in a vector with names
get.p.anova = function(obj) {
  ps = which(sapply(dimnames(obj)[[2]],function(x){substr(x,1,2)})=='Pr')
  if(length(ps) == 0) {writeLines('No p-values specified in anova'); return(invisible(NULL))}
  ret = obj[,ps]
  idx = which(!is.na(ret))
  ret = ret[idx]
  names(ret) = trim.whitespace(dimnames(obj)[[1]][idx])
  return(ret)
}
get.stat.anova = function(obj) {
  fs = which(dimnames(obj)[[2]] %in% c('F','F value'))
  if(length(fs) == 0) {
    # Check whether there is a chi-sq test to pull deviance from
    if(dimnames(obj)[[2]][5] == 'Pr(>Chi)') {
      ret = obj$Deviance
      idx = which(!is.na(ret))
      ret = ret[idx]
      names(ret) = trim.whitespace(dimnames(obj)[[1]][idx])
      return(ret)
    }
    else {writeLines('No valid test to use statistic'); return(invisible(NULL))}
  }
  ret = obj[,fs]
  idx = which(!is.na(ret))
  ret = ret[idx]
  names(ret) = dimnames(obj)[[1]][idx]
  return(ret)
}
get.df.anova = function(obj) {
  dfs = obj['Df']
  resi = which(sapply(dimnames(dfs)[[1]],function(x) {substr(x,1,5) == 'Resid'}))
  nms = dimnames(dfs)[[1]][-resi]
  df2 = dfs[resi,'Df']
  dfs = dfs[-resi,]
  names(dfs) = trim.whitespace(nms)
  dfs = data.frame(df1 = dfs)
  dfs$df2 = df2
  return(dfs)
}

# For lm, returns all p-values from the summary
# Alternately, set aov.type to True to return p-values from the anova
get.p.lm = function(obj, aov.type = FALSE) {
  if(aov.type) {return(get.p(anova(obj)))}
  cf = summary(obj)$coefficients
  d = dim(cf)
  return(cf[1:d[1],d[2]])
}
# Returns t-stats from summary
get.stat.lm = function(obj, omnibus = FALSE) {
  if(omnibus) {ret = summary(obj)$fstatistic[1];names(ret)='F';return(ret)}
  cf = summary(obj)$coefficients
  d = dim(cf)
  return(cf[1:d[1],(d[2]-1)])
}
get.df.lm = function(obj, aov.type = FALSE) {
  if(aov.type) {return(get.df(anova(obj)))}
  ret = summary(obj)$fstatistic[2:3]
  names(ret) = c('df1','df2')
  return(ret)
}
get.ci.lm = function(obj,conf.int = 0.95) {
  vardf = get.df(obj)[2]
  tlim = qt(1-(1-conf.int)/2,vardf)
  cfs = summary(obj)$coefficients[,1:2]
  ret = cbind(cfs[,1],cfs[,1]-tlim*cfs[,2],cfs[,1]+tlim*cfs[,2])
  dimnames(ret)[[2]] = c('Estimate','Lower','Upper')
  attr(ret,'conf.level') = conf.int
  return(ret)
}

# For aov, returns like anova
get.p.aov = function(obj) {
  return(get.p(summary(obj)[[1]]))
}
get.stat.aov = function(obj) {
  return(get.stat(summary(obj)[[1]]))
}
get.df.aov = function(obj) {return(get.df(summary(obj)[[1]]))}

# For aovlist (output of aov with Error), concatenate all of the lists
get.p.aovlist = function(obj) {
  ret = NULL
  for(o in obj) {
    if(o$df.residual > 0) {ret = c(ret,get.p(o))}
  }
  return(ret)
}
get.stat.aovlist = function(obj) {
  ret = NULL
  for(o in obj) {
    if(o$df.residual > 0) {ret = c(ret,get.stat(o))}
  }
  return(ret)
}
get.df.aovlist = function(obj) {
  ret = NULL
  for(o in obj) {
    if(o$df.residual > 0) {ret = rbind(ret,get.df(o))}
  }
  return(ret)
}

get.p.TukeyHSD = function(obj) {
  nmlist = NULL
  plist = NULL
  for(i in 1:length(obj)) {
    nmlist = c(nmlist,dimnames(obj[[i]])[[1]])
    plist = c(plist,obj[[i]][,4])
  }
  names(plist) = nmlist
  return(plist)
}

z.test = function(x, y = NULL, sigma = NULL, alternative = "two.sided",mu = 0, conf.level = 0.95) {
  if(is.null(sigma)) {writeLines('A sigma argument is required for z-tests'); return(invisible(NULL))}
  match.arg(alternative,choices = c('two.sided','less','greater'))
  x.mn = mean(x)
  y.mn = ifelse(is.null(y),0,mean(y))
  
  stderr = sigma/sqrt(length(x))
  diff = x.mn - y.mn
  conf.mult = -qnorm((1-conf.level)/2)
  conf.top = diff + conf.mult*stderr
  conf.bottom = diff - conf.mult*stderr
  
  z.sc = (diff - mu)/stderr
  if(alternative == 'two.sided') {pval = (1-pnorm(abs(z.sc)))*2}
  else if(alternative == 'less') {pval = pnorm(z.sc)}
  else if(alternative == 'greater') {pval = 1-pnorm(z.sc)}
  
  attr(z.sc,'names') = 'z'
  par = NA
  attr(par,'names') = 'df'
  ci = c(conf.bottom,conf.top)
  attr(ci,'conf.level') = conf.level
  xnm = deparse(substitute(x))
  if(is.null(y)) {
    estimate = x.mn
    attr(estimate,'names') = 'mean of x'
    attr(mu,'names') = 'mean'
    meth = 'One Sample z-test'
    dn = xnm
  } else {
    estimate = c(x.mn,y.mn)
    attr(estimate,'names') = c('mean of x','mean of y')
    attr(mu,'names') = 'difference in means'
    meth = 'Two Sample z-test'
    ynm = deparse(substitute(y))
    dn = paste(xnm,'and',ynm)
  }
  
  tst = list(statistic = z.sc, parameter = par, p.value = pval,
            conf.int = ci, estimate = estimate, null.value = mu, alternative = alternative,
            method = meth, data.name = dn)
  class(tst) = 'htest'
  return(tst)
}